@@ -7,11 +7,10 @@ from collections import deque
 from datetime import datetime
 from functools import partial
 import logging
-from typing import TYPE_CHECKING, Any, Callable, Deque, Optional, Type
+from typing import TYPE_CHECKING, Any, Callable, Deque, Optional, Type, TypeVar, cast
 
-from chip.ChipDeviceCtrl import ChipDeviceController
+from chip.ChipDeviceCtrl import CommissionableNode
 from chip.clusters import Attribute, ClusterCommand
-from chip.discovery import FilterType as DiscoveryFilterType
 from chip.exceptions import ChipStackError
 
 from ..common.helpers.api import api_command
@@ -22,7 +21,6 @@ from ..common.models.error import (
     NodeInterviewFailed,
     NodeNotExists,
     NodeNotResolving,
-    SDKCommandFailed,
 )
 from ..common.models.events import EventType
 from ..common.models.node import MatterAttribute, MatterNode
@@ -32,6 +30,8 @@ if TYPE_CHECKING:
 
     from .server import MatterServer
 
+_T = TypeVar("_T")
+
 DATA_KEY_NODES = "nodes"
 DATA_KEY_LAST_NODE_ID = "last_node_id"
 
@@ -45,30 +45,32 @@ class MatterDeviceController:
         self,
         server: MatterServer,
     ):
+        """Initialize the device controller."""
         self.server = server
         # Instantiate the underlying ChipDeviceController instance on the Fabric
-        self.chip_controller: ChipDeviceController = (
-            server.stack.fabric_admin.NewController()
-        )
+        self.chip_controller = server.stack.fabric_admin.NewController()
         # we keep the last events in memory so we can include them in the diagnostics dump
         self.event_history: Deque[Attribute.EventReadResult] = deque(maxlen=25)
         self._subscriptions: dict[int, Attribute.SubscriptionTransaction] = {}
         self._nodes: dict[int, MatterNode] = {}
         self.wifi_credentials_set: bool = False
         self.thread_credentials_set: bool = False
+        self.compressed_fabric_id: int | None = None
 
     @property
     def fabric_id(self) -> int:
         """Return Fabric ID."""
-        return self.chip_controller.fabricId
+        return cast(int, self.chip_controller.fabricId)
 
-    @property
-    def compressed_fabric_id(self) -> int:
-        """Return unique identifier for this initialized fabric."""
-        return self.chip_controller.GetCompressedFabricId()
+    async def initialize(self) -> None:
+        """Async initialize of controller."""
+        self.compressed_fabric_id = await self._call_sdk(
+            self.chip_controller.GetCompressedFabricId
+        )
+        LOGGER.debug("CHIP Device Controller Initialized")
 
     async def start(self) -> None:
-        """Async initialize of controller."""
+        """Handle logic on controller start."""
         # load nodes from persistent storage
         nodes_data = self.server.storage.get(DATA_KEY_NODES, {})
         for node_id_str, node_dict in nodes_data.items():
@@ -81,11 +83,11 @@ class MatterDeviceController:
             except NodeNotResolving:
                 LOGGER.warning("Node %s is not resolving, skipping...", node_id)
         # create task to check for nodes that need any re(interviews)
-        self.server.loop.create_task(self._check_interviews())
-        LOGGER.debug("CHIP Device Controller Initialized")
+        asyncio.create_task(self._check_interviews())
+        LOGGER.debug("Loaded %s nodes", len(self._nodes))
 
     async def stop(self) -> None:
-        """ "Handle logic on server stop."""
+        """Handle logic on server stop."""
         # unsubscribe all node subscriptions
         for sub in self._subscriptions.values():
             await self._call_sdk(sub.Shutdown)
@@ -114,22 +116,15 @@ class MatterDeviceController:
         """
         node_id = self._get_next_node_id()
 
-        # TODO TEMP !!!
-        # The call to CommissionWithCode returns early without waiting ?!
-        # This is most likely a bug in the SDK or its python wrapper
-        # success = await self._call_sdk(
-        #     self.chip_controller.CommissionWithCode,
-        #     setupPayload=code,
-        #     nodeid=node_id,
-        # )
-        # if not success:
-        #     raise NodeCommissionFailed(f"CommissionWithCode failed for node {node_id}")
-        await self._call_sdk(
+        success = await self._call_sdk(
             self.chip_controller.CommissionWithCode,
             setupPayload=code,
             nodeid=node_id,
         )
-        await asyncio.sleep(60)
+        if not success:
+            raise NodeCommissionFailed(
+                f"Commission with code failed for node {node_id}"
+            )
 
         # full interview of the device
         await self.interview_node(node_id)
@@ -142,7 +137,7 @@ class MatterDeviceController:
     async def commission_on_network(
         self,
         setup_pin_code: int,
-        filter_type: DiscoveryFilterType = DiscoveryFilterType.NONE,
+        filter_type: int = 0,
         filter: Any = None,  # pylint: disable=redefined-builtin
     ) -> MatterNode:
         """
@@ -154,6 +149,7 @@ class MatterDeviceController:
         Returns full NodeInfo once complete.
         """
         node_id = self._get_next_node_id()
+
         success = await self._call_sdk(
             self.chip_controller.CommissionOnNetwork,
             nodeId=node_id,
@@ -162,7 +158,10 @@ class MatterDeviceController:
             filter=filter,
         )
         if not success:
-            raise NodeCommissionFailed(f"CommissionWithCode failed for node {node_id}")
+            raise NodeCommissionFailed(
+                f"Commission on network failed for node {node_id}"
+            )
+
         # full interview of the device
         await self.interview_node(node_id)
         # make sure we start a subscription for this newly added node
@@ -173,26 +172,22 @@ class MatterDeviceController:
     @api_command(APICommand.SET_WIFI_CREDENTIALS)
     async def set_wifi_credentials(self, ssid: str, credentials: str) -> None:
         """Set WiFi credentials for commissioning to a (new) device."""
-        error_code = await self._call_sdk(
+        await self._call_sdk(
             self.chip_controller.SetWiFiCredentials,
             ssid=ssid,
             credentials=credentials,
         )
 
-        if error_code != 0:
-            raise SDKCommandFailed("Set WiFi credentials failed.")
         self.wifi_credentials_set = True
 
     @api_command(APICommand.SET_THREAD_DATASET)
     async def set_thread_operational_dataset(self, dataset: str) -> None:
         """Set Thread Operational dataset in the stack."""
-        error_code = await self._call_sdk(
+        await self._call_sdk(
             self.chip_controller.SetThreadOperationalDataset,
             threadOperationalDataset=bytes.fromhex(dataset),
         )
 
-        if error_code != 0:
-            raise SDKCommandFailed("Set Thread credentials failed.")
         self.thread_credentials_set = True
 
     @api_command(APICommand.OPEN_COMMISSIONING_WINDOW)
@@ -223,7 +218,9 @@ class MatterDeviceController:
         return discriminator
 
     @api_command(APICommand.DISCOVER)
-    async def discover_commissionable_nodes(self):
+    async def discover_commissionable_nodes(
+        self,
+    ) -> CommissionableNode | list[CommissionableNode] | None:
         """Discover Commissionable Nodes (discovered on BLE or mDNS)."""
 
         result = await self._call_sdk(
@@ -325,7 +322,8 @@ class MatterDeviceController:
         def attribute_updated_callback(
             path: Attribute.TypedAttributePath,
             transaction: Attribute.SubscriptionTransaction,
-        ):
+        ) -> None:
+            assert self.server.loop is not None
             new_value = transaction.GetAttribute(path)
             LOGGER.debug("attribute updated -- %s - new value: %s", path, new_value)
             node = self._nodes[node_id]
@@ -347,8 +345,9 @@ class MatterDeviceController:
         def event_callback(
             data: Attribute.EventReadResult,
             transaction: Attribute.SubscriptionTransaction,
-        ):
+        ) -> None:
             # pylint: disable=unused-argument
+            assert self.server.loop is not None
             LOGGER.debug("received node event: %s", data)
             self.event_history.append(data)
             # TODO: This callback does not seem to fire ever or my test devices do not have events
@@ -358,15 +357,15 @@ class MatterDeviceController:
 
         def error_callback(
             chipError: int, transaction: Attribute.SubscriptionTransaction
-        ):
+        ) -> None:
             # pylint: disable=unused-argument, invalid-name
-            LOGGER.error("Got error fron node: %s", chipError)
+            LOGGER.error("Got error from node: %s", chipError)
 
         def resubscription_attempted(
             transaction: Attribute.SubscriptionTransaction,
             terminationError: int,
             nextResubscribeIntervalMsec: int,
-        ):
+        ) -> None:
             # pylint: disable=unused-argument, invalid-name
             LOGGER.debug(
                 "Previous subscription failed with Error: %s - re-subscribing in %s ms...",
@@ -375,7 +374,9 @@ class MatterDeviceController:
             )
             # TODO: update node status to unavailable
 
-        def resubscription_succeeded(transaction: Attribute.SubscriptionTransaction):
+        def resubscription_succeeded(
+            transaction: Attribute.SubscriptionTransaction,
+        ) -> None:
             # pylint: disable=unused-argument, invalid-name
             LOGGER.debug("Subscription succeeded")
             # TODO: update node status to available
@@ -389,19 +390,26 @@ class MatterDeviceController:
 
     def _get_next_node_id(self) -> int:
         """Return next node_id."""
-        next_node_id = self.server.storage.get(DATA_KEY_LAST_NODE_ID, 0) + 1
+        next_node_id = cast(int, self.server.storage.get(DATA_KEY_LAST_NODE_ID, 0)) + 1
         self.server.storage.set(DATA_KEY_LAST_NODE_ID, next_node_id, force=True)
         return next_node_id
 
-    async def _call_sdk(self, func: Callable, *args, **kwargs) -> Any:
+    async def _call_sdk(self, func: Callable[..., _T], *args: Any, **kwargs: Any) -> _T:
         """Call function on the SDK in executor and return result."""
-        return await self.server.loop.run_in_executor(
-            None,
-            partial(func, *args, **kwargs),
+        if self.server.loop is None:
+            raise RuntimeError("Server not started.")
+
+        return cast(
+            _T,
+            await self.server.loop.run_in_executor(
+                None,
+                partial(func, *args, **kwargs),
+            ),
         )
 
     async def _check_interviews(self) -> None:
         """Check if any nodes need to be (re)interviewed."""
+        assert self.server.loop is not None
         # Reinterview all nodes that have an outdated schema
         # and/or have been interviewed more than 30 days ago.
         for node in self._nodes.values():
@@ -411,14 +419,12 @@ class MatterDeviceController:
             ):
                 await self.interview_node(node.node_id)
         # reschedule self to run every hour
-        self.server.loop.call_later(
-            3600, self.server.loop.create_task, self._check_interviews()
-        )
+        self.server.loop.call_later(3600, asyncio.create_task, self._check_interviews())
 
     @staticmethod
     def _parse_attributes_from_read_result(
         node_id: int, attributes: dict[int, dict[Type, dict[Type, Any]]]
-    ) -> dict[int, MatterAttribute]:
+    ) -> dict[str, MatterAttribute]:
         """Parse attributes from ReadResult."""
         result = {}
         for endpoint, cluster_dict in attributes.items():
